__package__ = 'pywebio_battery'
__description__ = 'Utilities that help write PyWebIO apps quickly and easily.'
__url__ = 'https://pywebio.readthedocs.io/en/latest/battery.html'
__version__ = "0.2.0"
__version_info__ = (0, 2, 0, 0)
__author__ = 'WeiminWang'
__author_email__ = 'wang0.618@qq.com'
__license__ = 'MIT'
